@@ -1,9 +1,9 @@
-/* $Id: files.c,v 1.310 2006/10/13 16:35:57 dolorous Exp $ */
+/* $Id: files.c,v 1.322 2006/11/10 02:47:11 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
+ *   Copyright (C) 2005, 2006 David Lawrence Ramsey                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -954,13 +954,11 @@ char *get_full_path(const char *origpath)
 
     d_there = real_dir_from_tilde(origpath);
 
-    assert(d_there != NULL);
-
     /* If stat()ing d_there fails, assume that d_there refers to a new
      * file that hasn't been saved to disk yet.  Set path_only to TRUE
      * if d_there refers to a directory, and FALSE otherwise. */
-    path_only = stat(d_there, &fileinfo) == 0 &&
-	S_ISDIR(fileinfo.st_mode);
+    path_only = (stat(d_there, &fileinfo) != -1 &&
+	S_ISDIR(fileinfo.st_mode));
 
     /* If path_only is TRUE, make sure d_there ends in a slash. */
     if (path_only) {
@@ -988,8 +986,7 @@ char *get_full_path(const char *origpath)
 	if (!path_only)
 	    d_there_file = mallocstrcpy(NULL, last_slash + 1);
 
-	/* And remove the filename portion of the answer from
-	 * d_there. */
+	/* Remove the filename portion of the answer from d_there. */
 	null_at(&d_there, last_slash - d_there + 1);
 
 	/* Go to the path specified in d_there. */
@@ -1021,28 +1018,29 @@ char *get_full_path(const char *origpath)
 
 	    /* Finally, go back to the path specified in d_here,
 	     * where we were before.  We don't check for a chdir()
-	     * error, since we can do nothing then. */
+	     * error, since we can do nothing if we get one. */
 	    chdir(d_here);
 
 	    /* Free d_here, since we're done using it. */
 	    free(d_here);
 	}
-
-	/* At this point, if path_only is FALSE and d_there isn't NULL,
-	 * d_there contains the path portion of the answer and
-	 * d_there_file contains the filename portion of the answer.  If
-	 * this is the case, tack the latter onto the end of the former.
-	 * d_there will then contain the complete answer. */
-	if (!path_only && d_there != NULL) {
-	    d_there = charealloc(d_there, strlen(d_there) +
-		strlen(d_there_file) + 1);
-	    strcat(d_there, d_there_file);
- 	}
-
-	/* Free d_there_file, since we're done using it. */
-	free(d_there_file);
     }
 
+    /* At this point, if path_only is FALSE and d_there isn't NULL,
+     * d_there contains the path portion of the answer and d_there_file
+     * contains the filename portion of the answer.  If this is the
+     * case, tack the latter onto the end of the former.  d_there will
+     * then contain the complete answer. */
+    if (!path_only && d_there != NULL) {
+	d_there = charealloc(d_there, strlen(d_there) +
+		strlen(d_there_file) + 1);
+	strcat(d_there, d_there_file);
+    }
+
+    /* Free d_there_file, since we're done using it. */
+    if (d_there_file != NULL)
+	free(d_there_file);
+
     return d_there;
 }
 
@@ -1145,10 +1143,10 @@ void init_operating_dir(void)
  * so that tab completion will work. */
 bool check_operating_dir(const char *currpath, bool allow_tabcomp)
 {
-    /* The char *full_operating_dir is global for mem cleanup.  It
-     * should have already been initialized by init_operating_dir().
-     * Also, a relative operating directory path will only be handled
-     * properly if this is done. */
+    /* full_operating_dir is global for memory cleanup.  It should have
+     * already been initialized by init_operating_dir().  Also, a
+     * relative operating directory path will only be handled properly
+     * if this is done. */
 
     char *fullpath;
     bool retval = FALSE;
@@ -1763,7 +1761,7 @@ int do_writeout(bool exiting)
 
     ans = mallocstrcpy(NULL,
 #ifndef NANO_TINY
-	(openfile->mark_set && !exiting) ? "" :
+	(!exiting && openfile->mark_set) ? "" :
 #endif
 	openfile->filename);
 
@@ -1778,7 +1776,7 @@ int do_writeout(bool exiting)
 
 	backupstr = ISSET(BACKUP_FILE) ? _(" [Backup]") : "";
 
-	if (openfile->mark_set && !exiting)
+	if (!exiting && openfile->mark_set)
 	    msg = (append == PREPEND) ?
 		_("Prepend Selection to File") : (append == APPEND) ?
 		_("Append Selection to File") :
@@ -1869,42 +1867,59 @@ int do_writeout(bool exiting)
 		break;
 	    }
 #endif
-	    if (append == OVERWRITE && strcmp(answer,
-		openfile->filename) != 0) {
-		struct stat st;
 
-		if (!stat(answer, &st)) {
-		    i = do_yesno_prompt(FALSE,
-			_("File exists, OVERWRITE ? "));
-		    if (i == 0 || i == -1)
-			continue;
-		/* If we're using restricted mode, we aren't allowed to
-		 * change the name of a file once it has one, because
-		 * that would allow reading from or writing to files not
-		 * specified on the command line.  In this case, don't
-		 * bother showing the "Different Name" prompt. */
-		} else if (!ISSET(RESTRICTED) &&
+	    if (append == OVERWRITE) {
+		char *full_answer = get_full_path(answer);
+		char *full_filename = get_full_path(openfile->filename);
+		bool different_name = (strcmp(full_answer,
+			full_filename) != 0);
+		struct stat st;
+		bool name_exists;
+
+		free(full_filename);
+		free(full_answer);
+
+		name_exists = (stat(answer, &st) != -1);
+
+		if (different_name) {
+		    if (name_exists) {
+			/* If we're using restricted mode, we aren't
+			 * allowed to save a new file under the name of
+			 * an existing file. */
+			if (ISSET(RESTRICTED))
+			    continue;
+
+			i = do_yesno_prompt(FALSE,
+				_("File exists, OVERWRITE ? "));
+			if (i == 0 || i == -1)
+			    continue;
+		    /* If we're using restricted mode, we aren't allowed
+		     * to change the name of a file once it has one,
+		     * because that would allow reading from or writing
+		     * to files not specified on the command line. */
+		    } else if (!ISSET(RESTRICTED) &&
 			openfile->filename[0] != '\0'
 #ifndef NANO_TINY
 			&& (exiting || !openfile->mark_set)
 #endif
 			) {
-		    i = do_yesno_prompt(FALSE,
-			_("Save file under DIFFERENT NAME ? "));
-		    if (i == 0 || i == -1)
-			continue;
+			i = do_yesno_prompt(FALSE,
+				_("Save file under DIFFERENT NAME ? "));
+			if (i == 0 || i == -1)
+			    continue;
+		    }
 		}
 	    }
 
 #ifndef NANO_TINY
 	    /* Here's where we allow the selected text to be written to
 	     * a separate file.  If we're using restricted mode, this is
-	     * disabled since it allows reading from or writing to files
-	     * not specified on the command line. */
+	     * disabled, since it allows reading from or writing to
+	     * files not specified on the command line. */
 	    if (!ISSET(RESTRICTED) && !exiting && openfile->mark_set)
 		retval = write_marked_file(answer, NULL, FALSE, append);
 	    else
-#endif /* !NANO_TINY */
+#endif
 		retval = write_file(answer, NULL, FALSE, append, FALSE);
 
 	    break;
