@@ -1,9 +1,9 @@
-/* $Id: winio.c,v 1.574 2006/10/29 21:14:53 dolorous Exp $ */
+/* $Id: winio.c,v 1.576 2006/11/10 20:13:38 dolorous Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
+ *   Copyright (C) 2005, 2006 David Lawrence Ramsey                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -3124,12 +3124,10 @@ void do_replace_highlight(bool highlight, const char *word)
     if (highlight)
 	wattron(edit, reverse_attr);
 
-#ifdef HAVE_REGEX_H
-    /* This is so we can show zero-length regex matches. */
+    /* This is so we can show zero-length matches. */
     if (word_len == 0)
 	waddch(edit, ' ');
     else
-#endif
 	waddnstr(edit, word, actual_x(word, y));
 
     if (word_len > y)
@@ -3199,8 +3197,8 @@ void do_credits(void)
 	"",
 	"",
 	"",
-	"(c) 1999-2004 Chris Allegretta",
-	"(c) 2005-2006 David Lawrence Ramsey",
+	"(c) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta",
+	"(c) 2005, 2006 David Lawrence Ramsey",
 	"",
 	"",
 	"",
